@@ -186,7 +186,7 @@ static int get_conflicting_search_paths(const Options *op, char ***paths)
  *   - backing and removing up conflicting files
  *   - installing all the installable files
  *   - create any needed symbolic links
- *   - executing any nessary commands
+ *   - executing any necessary commands
  *   - running `ldconfig` and `depmod -aq`
  *
  * If we are only installing the kernel module, then we trim back all
@@ -365,19 +365,6 @@ CommandList *build_command_list(Options *op, Package *p)
     
     for (i = 0; i < p->num_entries; i++) {
         if (p->entries[i].caps.is_symlink) {
-            /* if it's a NEWSYM and the file already exists, don't add a command
-             * for it */
-            if (p->entries[i].type == FILE_TYPE_XMODULE_NEWSYM) {
-                struct stat buf;
-                if(!stat(p->entries[i].dst, &buf) || errno != ENOENT) {
-                    ui_expert(op, "Not creating a symlink from %s to %s "
-                                  "because a file already exists at that path "
-                                  "or the path is inaccessible.",
-                                  p->entries[i].dst, p->entries[i].target);
-                    continue;
-                }
-            }
-
             add_command(c, SYMLINK_CMD, p->entries[i].dst,
                         p->entries[i].target);
         }
@@ -421,12 +408,21 @@ CommandList *build_command_list(Options *op, Package *p)
      * <Nigel.Spowage@energis.com>
      */
     
-    if (!op->no_kernel_module) {
+    if (!op->no_kernel_module && !op->skip_depmod) {
         tmp = nvstrcat(op->utils[DEPMOD], " -a ", op->kernel_name, NULL);
         add_command(c, RUN_CMD, tmp);
         nvfree(tmp);
     }
-    
+
+    /*
+     * If systemd files were installed, run `systemctl daemon-reload`.
+     */
+    if (op->use_systemd == NV_OPTIONAL_BOOL_TRUE) {
+        tmp = nvstrcat(op->utils[SYSTEMCTL], " daemon-reload", NULL);
+        add_command(c, RUN_CMD, tmp);
+        nvfree(tmp);
+    }
+
     /* free the FileList */
     free_file_list(l);
 
@@ -912,11 +908,8 @@ ConflictingFileInfo *build_conflicting_file_list(Options *op, Package *p)
     }
 
     /* XXX always conflict with these files if OpenGL files will be installed
-     * libglamoregl.so: prevent X from loading libGL and libglx simultaneously
-     *                  (bug 1299091)
      * libGLwrapper.so: this library has an SONAME of libGL.so.1 (bug 74761) */
     if (!op->no_opengl_files) {
-        get_conflicting_file_info("libglamoregl.so", &cfList[index++]);
         get_conflicting_file_info("libGLwrapper.so", &cfList[index++]);
     }
 
@@ -929,7 +922,7 @@ ConflictingFileInfo *build_conflicting_file_list(Options *op, Package *p)
 
 
 /*
- * condense_file_list() - Take a FileList stucture and delete any
+ * condense_file_list() - Take a FileList structure and delete any
  * duplicate entries in the list.  This is a pretty brain dead
  * brute-force algorithm.
  */
