@@ -69,6 +69,12 @@ static const KernelModuleInfo optional_modules[] = {
          .disable_option = "no-drm",
          .option_offset = offsetof(Options, install_drm),
     },
+    {
+        .module_name = "nvidia-peermem",
+        .optional_module_dependee = "GPUDirect RDMA p2p memory sharing",
+        .disable_option = "no-peermem",
+        .option_offset = offsetof(Options, install_peermem),
+    },
 };
 
 /*
@@ -96,7 +102,7 @@ int install_from_cwd(Options *op)
     HookScriptStatus res;
 
     static const char* edit_your_xf86config =
-        "Please update your XF86Config or xorg.conf file as "
+        "Please update your xorg.conf file as "
         "appropriate; see the file /usr/share/doc/"
         "NVIDIA_GLX-1.0/README.txt for details.";
 
@@ -203,7 +209,7 @@ int install_from_cwd(Options *op)
                 "not remove existing NVIDIA kernel modules not part of "
                 "an earlier NVIDIA driver installation.  Please ensure "
                 "that an NVIDIA kernel module matching this driver version "
-                "is installed seperately.");
+                "is installed separately.");
 
         /* no_kernel_module should imply no DKMS */
 
@@ -229,25 +235,13 @@ int install_from_cwd(Options *op)
     
         if (!get_prefixes(op)) goto failed;
 
-        /* ask if we should install the OpenGL header files */
-
-        should_install_opengl_headers(op, p);
-
         /*
-         * select the appropriate TLS class, modifying the package as
-         * necessary.
-         */
-    
-        select_tls_class(op, p);
-
-        /*
-         * if the package contains any libGL.la or .desktop files,
+         * if the package contains any .desktop files,
          * process them (perform some search and replacing so
          * that they reflect the correct installation path, etc)
          * and add them to the package list (files to be installed).
          */
-        
-        process_libGL_la_files(op, p);
+
         process_dot_desktop_files(op, p);
 
 #if defined(NV_X86_64)
@@ -262,15 +256,18 @@ int install_from_cwd(Options *op)
 
     if (op->no_opengl_files) {
         remove_opengl_files_from_package(op, p);
-    } else {
-        select_glvnd(op, p);
+    }
+
+    if (op->no_wine_files) {
+        remove_wine_files_from_package(op, p);
     }
 
     /*
-     * now that we selected GLVND, generate the proper Vulkan ICD configuration
-     * file
+     * determine whether systemd files should be installed
      */
-    process_vulkan_icd_file(op, p);
+    if (op->use_systemd != NV_OPTIONAL_BOOL_TRUE) {
+        remove_systemd_files_from_package(op, p);
+    }
 
     /*
      * now that we have the installation prefixes, build the
@@ -545,13 +542,6 @@ precompiled_done:
          */
         
         if (!determine_kernel_source_path(op, p)) return FALSE;
-
-        /*
-         * make sure that the selected or default system compiler
-         * is compatible with the target kernel; the user may choose
-         * to override the check.
-         */
-        if (!check_cc_version(op, p)) return FALSE;
     
         /* and now, build the kernel interface */
         
@@ -636,6 +626,7 @@ static int has_separate_interface_file(char *name) {
         "nvidia-vgpu-vfio",
         "nvidia-uvm",
         "nvidia-drm",
+        "nvidia-peermem",
     };
 
     for (i = 0; i < ARRAY_LEN(no_interface_modules); i++) {
@@ -823,10 +814,22 @@ static Package *parse_manifest (Options *op)
     line++;
     p->kernel_module_build_directory = get_next_line(ptr, &ptr, manifest, len);
     if (!p->kernel_module_build_directory) goto invalid_manifest_file;
+
+    /*
+     * allow the kernel module build directory to be overridden from the command
+     * line
+     */
+
+    if (op->kernel_module_build_directory_override) {
+        nvfree(p->kernel_module_build_directory);
+        p->kernel_module_build_directory =
+            nvstrdup(op->kernel_module_build_directory_override);
+    }
+
     remove_trailing_slashes(p->kernel_module_build_directory);
 
     /*
-     * the eigth line is the directory containing precompiled kernel
+     * the eighth line is the directory containing precompiled kernel
      * interfaces
      */
 
@@ -927,24 +930,6 @@ static Package *parse_manifest (Options *op)
             op->compat32_files_packaged = TRUE;
         }
 
-        /* some libs/symlinks have a class field */
-
-        entry.tls_class = FILE_TLS_CLASS_NONE;
-
-        if (entry.caps.has_tls_class) {
-            nvfree(flag);
-            flag = read_next_word(c, &c);
-            if (!flag) goto entry_done;
-
-            if (strcmp(flag, "CLASSIC") == 0)
-                entry.tls_class = FILE_TLS_CLASS_CLASSIC;
-            else if (strcmp(flag, "NEW") == 0)
-                entry.tls_class = FILE_TLS_CLASS_NEW;
-            else {
-                goto entry_done;
-            }
-        }
-
         /* some file types have a path field, or inherit their paths */
 
         if (entry.caps.has_path) {
@@ -997,22 +982,6 @@ static Package *parse_manifest (Options *op)
             entry.target = NULL;
         }
 
-        /* some files are exclusive to GLVND or non-GLVND installations */
-
-        if (entry.caps.glvnd_select) {
-            char *type = read_next_word(c, &c);
-
-            if (!type) goto invalid_manifest_file;
-
-            if (strcmp(type, "GLVND") == 0) {
-                entry.glvnd = FILE_GLVND_GLVND_ONLY;
-            } else if (strcmp(type, "NON_GLVND") == 0) {
-                entry.glvnd = FILE_GLVND_NON_GLVND_ONLY;
-            } else {
-                goto invalid_manifest_file;
-            }
-        }
-
         /*
          * as a convenience for later, set the 'name' pointer to
          * the basename contained in 'file' (ie the portion of
@@ -1031,9 +1000,7 @@ static Package *parse_manifest (Options *op)
                           entry.target,
                           entry.dst,
                           entry.type,
-                          entry.tls_class,
                           entry.compat_arch,
-                          entry.glvnd,
                           entry.mode);
 
         entry_success = TRUE;
@@ -1093,9 +1060,7 @@ void add_package_entry(Package *p,
                        char *target,
                        char *dst,
                        PackageEntryFileType type,
-                       PackageEntryFileTlsClass tls_class,
                        PackageEntryFileCompatArch compat_arch,
-                       PackageEntryFileGLVND glvnd,
                        mode_t mode)
 {
     int n;
@@ -1114,11 +1079,9 @@ void add_package_entry(Package *p,
     p->entries[n].target      = target;
     p->entries[n].dst         = dst;
     p->entries[n].type        = type;
-    p->entries[n].tls_class   = tls_class;
     p->entries[n].mode        = mode;
     p->entries[n].caps        = get_file_type_capabilities(type);
     p->entries[n].compat_arch = compat_arch;
-    p->entries[n].glvnd       = glvnd;
 
     if (stat(p->entries[n].file, &stat_buf) != -1) {
         p->entries[n].inode = stat_buf.st_ino;
@@ -1284,7 +1247,7 @@ static int assisted_module_signing(Options *op, Package *p)
 
             /* Determine what hashing algorithm to use for the generated X.509
              * certificate. XXX The default is to use the same hash that is
-             * used for signing modules; the two hashes are actually orthognal
+             * used for signing modules; the two hashes are actually orthogonal
              * to each other, but by choosing the module signing hash we are
              * guaranteed that the chosen hash will be built into the kernel.
              */
@@ -1426,7 +1389,7 @@ generate_done:
                                       "delete the private signing key?");
 
         /* Get the fingerprint of the X.509 certificate. We already used 
-           openssl to create a keypair at this point, so we know we have it;
+           openssl to create a key pair at this point, so we know we have it;
            otherwise, we would have already returned by now. */
         cmdline = nvstrcat(op->utils[OPENSSL], " x509 -noout -fingerprint ",
                            "-inform DER -in ", op->module_signing_public_key,
@@ -1480,9 +1443,7 @@ generate_done:
                           NULL, /* target */
                           NULL, /* dst */
                           FILE_TYPE_MODULE_SIGNING_KEY,
-                          FILE_TLS_CLASS_NONE,
                           FILE_COMPAT_ARCH_NONE,
-                          FILE_GLVND_DONT_CARE,
                           0444);
 
         ui_message(op, "An X.509 certificate containing the public signing "
@@ -1513,9 +1474,7 @@ generate_done:
                               NULL, /* target */
                               NULL, /* dst */
                               FILE_TYPE_MODULE_SIGNING_KEY,
-                              FILE_TLS_CLASS_NONE,
                               FILE_COMPAT_ARCH_NONE,
-                              FILE_GLVND_DONT_CARE,
                               0400);
 
             ui_message(op, "The private signing key will be installed to %s/%s. "
