@@ -23,7 +23,6 @@
 #include <sys/utsname.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/sysctl.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <dirent.h>
@@ -50,9 +49,6 @@ static char *default_kernel_source_path(Options *op);
 static char *find_module_substring(char *string, const char *substring);
 static int check_for_loaded_kernel_module(Options *op, const char *);
 static void check_for_warning_messages(Options *op);
-static int sanity_check(Options *op, const char *dir,
-                        const char *sanity_check_name,
-                        const char *conftest_name);
 
 static PrecompiledInfo *scan_dir(Options *op, Package *p,
                                  const char *directory_name,
@@ -70,6 +66,7 @@ static void load_kernel_module_quiet(Options *op, const char *module_name);
 static void modprobe_remove_kernel_module_quiet(Options *op, const char *name);
 static int kernel_configuration_conflict(Options *op, Package *p,
                                          int target_system_checks);
+static int check_cc_version(Options *op, Package *p);
 
 /*
  * Message text that is used by several error messages.
@@ -159,7 +156,7 @@ int determine_kernel_module_installation_path(Options *op)
 static int run_conftest(Options *op, const char *dir, const char *args,
                         char **result)
 {
-    char *cmd, *arch;
+    char *cmd, *arch, *kernel_source_path, *kernel_output_path;
     int ret;
 
     if (result) {
@@ -171,12 +168,23 @@ static int run_conftest(Options *op, const char *dir, const char *args,
         return FALSE;
     }
 
+    /* Some conftests don't require kernel source/output paths;
+     * if run_conftest() is run early enough, these may not be
+     * set yet, so use a placeholder string instead of NULL to
+     * prevent premature termination of nvstrcat() below. */
+    kernel_source_path = kernel_output_path = "DIRECTORY_PLACEHOLDER";
+    if (op->kernel_source_path) {
+        kernel_source_path = op->kernel_source_path;
+    }
+    if (op->kernel_output_path) {
+        kernel_output_path = op->kernel_output_path;
+    }
+
     cmd = nvstrcat("sh \"", dir, "/conftest.sh\" \"",
-                   op->utils[CC], "\" \"",
                    op->utils[CC], "\" \"",
                    arch, "\" \"",
-                   op->kernel_source_path, "\" \"",
-                   op->kernel_output_path, "\" ",
+                   kernel_source_path, "\" \"",
+                   kernel_output_path, "\" ",
                    args, NULL);
 
     ret = run_command(op, cmd, result, FALSE, 0, TRUE);
@@ -198,7 +206,7 @@ static int run_conftest(Options *op, const char *dir, const char *args,
 int determine_kernel_source_path(Options *op, Package *p)
 {
     char *result;
-    char *source_files[2], *source_path;
+    char *version_h, *uapi_version_h;
     int ret, count = 0;
     
     /* determine the kernel source path */
@@ -286,49 +294,30 @@ int determine_kernel_source_path(Options *op, Package *p)
 
     if (!determine_kernel_output_path(op)) return FALSE;
 
-    ret = run_conftest(op, p->kernel_module_build_directory, "get_uname",
-                       &result);
+    /* check to make sure that either include/linux/version.h or
+     * include/generated/uapi/linux/version.h is present in the output dir */
+
+#define VERSION_H_PATH "/include/linux/version.h"
+#define UAPI_VERSION_H_PATH "/include/generated/uapi/linux/version.h"
+
+    version_h = nvstrcat(op->kernel_output_path, VERSION_H_PATH, NULL);
+    uapi_version_h = nvstrcat(op->kernel_output_path, UAPI_VERSION_H_PATH,
+                              NULL);
+
+    ret = access(version_h, F_OK) == 0 || access(uapi_version_h, F_OK) == 0;
+
+    free(version_h);
+    free(uapi_version_h);
 
     if (!ret) {
-        ui_error(op, "Unable to determine the version of the kernel "
-                 "sources located in '%s'.  %s",
-                 op->kernel_source_path, install_your_kernel_source);
-        free(result);
+        ui_error(op, "Neither the '" VERSION_H_PATH "' nor the '"
+                 UAPI_VERSION_H_PATH "' kernel header file exists.  "
+                 "The most likely reason for this is that the kernel "
+                 "source files in '%s' have not been configured.",
+                 op->kernel_output_path);
         return FALSE;
     }
 
-    if (strncmp(result, "2.4", 3) == 0) {
-        source_files[0] = nvstrcat(op->kernel_source_path,
-                                   "/include/linux/version.h", NULL);
-        source_files[1] = NULL;
-        source_path = op->kernel_source_path;
-    } else {
-        source_files[0] = nvstrcat(op->kernel_output_path,
-                                   "/include/linux/version.h", NULL);
-        source_files[1] = nvstrcat(op->kernel_output_path,
-                                   "/include/generated/uapi/linux/version.h",
-                                   NULL);
-        source_path = op->kernel_output_path;
-    }
-    free(result);
-
-    if (access(source_files[0], F_OK) != 0) {
-        if (!source_files[1]) {
-            ui_error(op, "The kernel header file '%s' does not exist.  "
-                     "The most likely reason for this is that the kernel "
-                     "source files in '%s' have not been configured.",
-                     source_files[0], source_path);
-            return FALSE;
-        } else if (access(source_files[1], F_OK) != 0) {
-            ui_error(op, "Neither the '%s' nor the '%s' kernel header "
-                     "file exists.  The most likely reason for this "
-                     "is that the kernel source files in '%s' have not been "
-                     "configured.",
-                     source_files[0], source_files[1], source_path);
-            return FALSE;
-        }
-    }
-
     /* OK, we seem to have a path to a configured kernel source tree */
     
     ui_log(op, "Kernel source path: '%s'\n", op->kernel_source_path);
@@ -506,7 +495,7 @@ static int attach_signature(Options *op, Package *p,
 int unpack_kernel_modules(Options *op, Package *p, const char *build_directory,
                           const PrecompiledFileInfo *fileInfo)
 {
-    char *cmd, *result;
+    char *cmd;
     int ret;
     uint32 attrmask;
 
@@ -533,7 +522,7 @@ int unpack_kernel_modules(Options *op, Package *p, const char *build_directory,
                    fileInfo->target_directory, "/", fileInfo->core_object_name,
                    NULL);
 
-    ret = run_command(op, cmd, &result, TRUE, 0, TRUE);
+    ret = run_command(op, cmd, NULL, TRUE, 0, TRUE);
 
     free(cmd);
 
@@ -961,13 +950,19 @@ int build_kernel_interfaces(Options *op, Package *p,
 
     /* run sanity checks */
     for (i = 0; i < ARRAY_LEN(sanity_checks); i++) {
-        if (!sanity_check(op, builddir,
-                          sanity_checks[i].sanity_check_name,
-                          sanity_checks[i].conftest_name)) {
+        if (!conftest_sanity_check(op, builddir,
+                                   sanity_checks[i].sanity_check_name,
+                                   sanity_checks[i].conftest_name)) {
             return FALSE;
         }
     }
 
+    /* run cc_version_check separately to allow check_cc_version() to set
+     * IGNORE_CC_MISMATCH if needed */
+    if (!check_cc_version(op, p)) {
+        return FALSE;
+    }
+
     ui_log(op, "Cleaning kernel module build directory.");
     run_make(op, p, builddir, "clean", NULL, NULL, 0);
     ret = run_make(op, p, builddir, "", NULL, "Building kernel modules", 25);
@@ -1086,9 +1081,9 @@ void check_for_warning_messages(Options *op)
 #define PRINTK_LOGLEVEL_KERN_ALERT 1
 
 /*
- * Attempt to set the printk loglevel, first using the /proc/sys interface,
- * and falling back to the deprecated sysctl if that fails. Pass the previous
- * loglevel back to the caller and return TRUE on success, or FALSE on failure.
+ * Attempt to set the printk loglevel using the /proc/sys interface.
+ * Pass the previous loglevel back to the caller and return TRUE on success,
+ * or FALSE on failure.
  */
 static int set_loglevel(int level, int *old_level)
 {
@@ -1098,6 +1093,9 @@ static int set_loglevel(int level, int *old_level)
     fp = fopen("/proc/sys/kernel/printk", "r+");
     if (fp) {
         if (!old_level || fscanf(fp, "%d ", old_level) == 1) {
+            /* Use a dynamic buffer for the string: the kernel does not range
+             * check the loglevel, so the value reported by the procfs file
+             * may have an unknown number of digits. */
             char *strlevel = nvasprintf("%d", level);
 
             fseek(fp, 0, SEEK_SET);
@@ -1110,23 +1108,6 @@ static int set_loglevel(int level, int *old_level)
         fclose(fp);
     }
 
-    if (!loglevel_set) {
-        /*
-         * Explicitly initialize the value of len, even though it looks like the
-         * syscall should do that, since in practice it doesn't always actually
-         * set the value of the pointed-to length parameter.
-         */
-        size_t len = sizeof(int);
-        int name[] = { CTL_KERN, KERN_PRINTK };
-
-        if (!old_level ||
-            sysctl(name, ARRAY_LEN(name), old_level, &len, NULL, 0) == 0) {
-            if (sysctl(name, ARRAY_LEN(name), NULL, 0, &level, len) == 0) {
-                loglevel_set = TRUE;
-            }
-        }
-    }
-
     return loglevel_set;
 }
 
@@ -1332,8 +1313,16 @@ int test_kernel_modules(Options *op, Package *p)
 {
     char *cmd = NULL, *data = NULL;
     int ret = FALSE, i;
-    const char *depmods[] = { "i2c-core", "drm", "drm-kms-helper", "vfio_mdev",
-                              "ipmi_msghandler" };
+    const char *depmods[] = {
+        "i2c-core",
+        "drm",
+        "drm-kms-helper",
+        "vfio_mdev",
+        "vfio",
+        "mdev",
+        "video",
+        "backlight",
+    };
 
     /* 
      * If we're building/installing for a different kernel, then we
@@ -1359,10 +1348,20 @@ int test_kernel_modules(Options *op, Package *p)
 
     for (i = 0; i < p->num_kernel_modules; i++) {
         int module_success = FALSE, insmod_ret;
-        char *module_path = nvstrcat(p->kernel_module_build_directory, "/",
-                                     p->kernel_modules[i].module_filename,
-                                     NULL);
         const char *module_opts = "";
+        char *module_path;
+
+        /*
+         * nvidia-peermem depends on out-of-tree kernel modules,
+         * so we can't reliably test-insmod it here
+         */
+        if (strcmp(p->kernel_modules[i].module_name, "nvidia-peermem") == 0) {
+            continue;
+        }
+
+        module_path = nvstrcat(p->kernel_module_build_directory, "/",
+                               p->kernel_modules[i].module_filename,
+                               NULL);
 
         if (strcmp(p->kernel_modules[i].module_name, "nvidia") == 0) {
             module_opts = "NVreg_DeviceFileUID=0 NVreg_DeviceFileGID=0 "
@@ -1495,23 +1494,6 @@ static void modprobe_remove_kernel_module_quiet(Options *op, const char *name)
     modprobe_helper(op, name, TRUE, TRUE);
 }
 
-/*
- * Attempt to load all kernel modules that are part of the Package. Returns
- * the number of successfully loaded kernel modules.
- */
-int load_kernel_modules(Options *op, Package *p)
-{
-    int i;
-
-    for (i = 0; i < p->num_kernel_modules; i++) {
-        if (!load_kernel_module(op, p->kernel_modules[i].module_name)) {
-            break;
-        }
-    }
-
-    return i;
-}
-
 
 
 /*
@@ -1580,7 +1562,7 @@ int check_for_unloaded_kernel_module(Options *op)
                      "to upgrade your driver.  If no GPU-based programs are "
                      "running, you know that your kernel supports module "
                      "unloading, and you still receive this message, then an "
-                     "error may have occured that has corrupted an NVIDIA "
+                     "error may have occurred that has corrupted an NVIDIA "
                      "kernel module's usage count, for which the simplest "
                      "remedy is to reboot your computer.",
                      conflicting_kernel_modules[n]);
@@ -1713,32 +1695,35 @@ PrecompiledInfo *find_precompiled_kernel_interface(Options *op, Package *p)
  * the user specified via the --kernel-name option, or `uname -r`.
  */
 
-char __kernel_name[256];
-
 char *get_kernel_name(Options *op)
 {
+    static char kernel_name[256];
     struct utsname uname_buf;
 
-    __kernel_name[0] = '\0';
+    if (!kernel_name[0]) {
+        if (uname(&uname_buf) == -1) {
+            static int uname_failed;
 
-    if (uname(&uname_buf) == -1) {
-        ui_warn(op, "Unable to determine the version of the running kernel "
-                "(%s).", strerror(errno));
-    } else {
-        strncpy(__kernel_name, uname_buf.release, sizeof(__kernel_name));
-        __kernel_name[sizeof(__kernel_name) - 1] = '\0';
+            if (!uname_failed) {
+                ui_warn(op, "Unable to determine the version of the running "
+                            "kernel (%s).", strerror(errno));
+                uname_failed = TRUE;
+            }
+        } else {
+            strncpy(kernel_name, uname_buf.release, sizeof(kernel_name) - 1);
+        }
     }
 
     if (op->kernel_name) {
-        if (strcmp(op->kernel_name, __kernel_name) != 0) {
+        if (strcmp(op->kernel_name, kernel_name) != 0) {
             /* Don't load kernel modules built against a non-running kernel */
             op->skip_module_load = TRUE;
         }
         return op->kernel_name;
     }
 
-    if (__kernel_name[0]) {
-        return __kernel_name;
+    if (kernel_name[0]) {
+        return kernel_name;
     }
 
     return NULL;
@@ -2022,7 +2007,7 @@ static int substring_is_isolated(const char *substring, const char *string,
  * kernel module is loaded; FALSE if it is not.
  *
  * Be sure to check that the character following the kernel module
- * name is a space (to avoid getting false positivies when the given
+ * name is a space (to avoid getting false positives when the given
  * kernel module name is contained within another kernel module name.
  */
 
@@ -2088,7 +2073,7 @@ int rmmod_kernel_module(Options *op, const char *module_name)
  * currently running kernel.
  */
 
-int check_cc_version(Options *op, Package *p)
+static int check_cc_version(Options *op, Package *p)
 {
     char *result;
     int ret;
@@ -2137,13 +2122,14 @@ int check_cc_version(Options *op, Package *p)
 
 
 /*
- * sanity_check() - run the given sanity check conftest; if the test fails,
- * print the error message from the test. Return the status from the test.
+ * conftest_sanity_check() - run the given sanity check conftest; if the test
+ * fails, print the error message from the test. Return the status from the
+ * test.
  */
 
-static int sanity_check(Options *op, const char *dir,
-                        const char *sanity_check_name,
-                        const char *conftest_name)
+int conftest_sanity_check(Options *op, const char *dir,
+                          const char *sanity_check_name,
+                          const char *conftest_name)
 {
     char *result, *conftest_args;
     int ret;
@@ -2155,7 +2141,8 @@ static int sanity_check(Options *op, const char *dir,
     nvfree(conftest_args);
 
     if (!ret && result) {
-        ui_error(op, "%s", result);
+        ui_error(op, "The %s sanity check failed:\n\n%s",
+                 sanity_check_name, result);
     }
 
     nvfree(result);
