@@ -43,6 +43,7 @@
 #include "sanity.h"
 #include "option_table.h"
 #include "msg.h"
+#include "manifest.h"
 
 static void print_version(void);
 static void print_help(const char* name, int is_uninstall, int advanced);
@@ -53,12 +54,10 @@ static void print_help(const char* name, int is_uninstall, int advanced);
  * print_version() - print the current nvidia-installer version number
  */
 
-extern const char *pNV_ID;
-
 static void print_version(void)
 {
     nv_info_msg(NULL, "");
-    nv_info_msg(NULL, "%s", pNV_ID);
+    nv_info_msg(NULL, "%s", NV_ID_STRING);
     nv_info_msg(TAB, "The NVIDIA Software Installer for Unix/Linux.");
     nv_info_msg(NULL, "");
     nv_info_msg(TAB, "This program is used to install, upgrade and uninstall "
@@ -129,7 +128,6 @@ static Options *load_default_options(void)
     op->tmpdir = get_tmpdir(op);
 
     op->logging = TRUE; /* log by default */
-    op->opengl_headers = FALSE; /* do not install our GL headers by default */
     op->nvidia_modprobe = TRUE;
     op->run_nvidia_xconfig = FALSE;
     op->selinux_option = SELINUX_DEFAULT;
@@ -141,18 +139,47 @@ static Options *load_default_options(void)
     op->check_for_alternate_installs = TRUE;
     op->install_uvm = TRUE;
     op->install_drm = TRUE;
-    op->glvnd_glx_client = TRUE;
-    op->glvnd_egl_client = TRUE;
+    op->install_peermem = TRUE;
     op->install_compat32_libs = NV_OPTIONAL_BOOL_DEFAULT;
     op->install_libglx_indirect = NV_OPTIONAL_BOOL_DEFAULT;
     op->install_libglvnd_libraries = NV_OPTIONAL_BOOL_DEFAULT;
     op->external_platform_json_path = DEFAULT_EGL_EXTERNAL_PLATFORM_JSON_PATH;
+    op->skip_depmod = FALSE;
+    op->use_systemd = NV_OPTIONAL_BOOL_DEFAULT;
 
     return op;
 
 } /* load_default_options() */
 
 
+/* Parse the file type and destination out from the argument to the
+ * --override-file-type-destination command line option. Return TRUE if
+ * parsing was successful, or FALSE on a parse error. */
+static int assign_file_type_override(Options *op, char *optarg)
+{
+    PackageEntryFileCapabilities dummy;
+    PackageEntryFileType type;
+    char *split;
+
+    /* Terminate the string at the first ':' to get the file type, and
+     * advance to the character after the ':' to get the destination. */
+    split = strchr(optarg, ':');
+    if (split == NULL) {
+        return FALSE;
+    }
+    split[0] = '\0';
+    split++;
+
+    type = parse_manifest_file_type(optarg, &dummy);
+    if (type == FILE_TYPE_NONE) {
+        return FALSE;
+    }
+
+    op->file_type_destination_overrides[type] = nvstrdup(split);
+
+    return TRUE;
+}
+
 
 /*
  * parse_commandline() - Populate the Options structure with
@@ -250,6 +277,10 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->opengl_prefix = strval; break;
         case OPENGL_LIBDIR_OPTION:
             op->opengl_libdir = strval; break;
+        case WINE_PREFIX_OPTION:
+            op->wine_prefix = strval; break;
+        case WINE_LIBDIR_OPTION:
+            op->wine_libdir = strval; break;
 #if defined(NV_X86_64)
         case COMPAT32_CHROOT_OPTION:
             op->compat32_chroot = strval; break;
@@ -295,34 +326,18 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             break;
         case TMPDIR_OPTION:
             op->tmpdir = strval; break;
-        case OPENGL_HEADERS_OPTION:
-            op->opengl_headers = TRUE; break;
         case NO_NVIDIA_MODPROBE_OPTION:
             op->nvidia_modprobe = FALSE; break;
-#if defined(NV_TLS_TEST)
         case FORCE_TLS_OPTION:
-            if (strcasecmp(strval, "new") == 0)
-                op->which_tls = FORCE_NEW_TLS;
-            else if (strcasecmp(strval, "classic") == 0)
-                op->which_tls = FORCE_CLASSIC_TLS;
-            else {
-                nv_error_msg("Invalid parameter for '--force-tls'");
-                goto fail;
-            }
+            /* This option is no longer used; ignore it. */
+            nv_warning_msg("The '--force-tls' option is deprecated:  "
+                           "nvidia-installer will ignore this option.");
             break;
-#if defined(NV_X86_64)
         case FORCE_TLS_COMPAT32_OPTION:
-            if (strcasecmp(strval, "new") == 0)
-                op->which_tls_compat32 = FORCE_NEW_TLS;
-            else if (strcasecmp(strval, "classic") == 0)
-                op->which_tls_compat32 = FORCE_CLASSIC_TLS;
-            else {
-                nv_error_msg("Invalid parameter for '--force-tls-compat32'");
-                goto fail;
-            }
+            /* This option is no longer used; ignore it. */
+            nv_warning_msg("The '--force-tls-compat32' option is deprecated:  "
+                           "nvidia-installer will ignore this option.");
             break;
-#endif /* NV_X86_64 */
-#endif /* NV_TLS_TEST */
         case SANITY_OPTION:
             op->sanity = TRUE;
             break;
@@ -390,6 +405,9 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case NO_OPENGL_FILES_OPTION:
             op->no_opengl_files = TRUE;
             break;
+        case NO_WINE_FILES_OPTION:
+            op->no_wine_files = TRUE;
+            break;
         case KERNEL_MODULE_SOURCE_PREFIX_OPTION:
             op->kernel_module_src_prefix = strval;
             break;
@@ -435,6 +453,9 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case NO_DRM_OPTION:
             op->install_drm = FALSE;
         break;
+        case NO_PEERMEM_OPTION:
+            op->install_peermem = FALSE;
+        break;
         case NO_CHECK_FOR_ALTERNATE_INSTALLS_OPTION:
             op->check_for_alternate_installs = FALSE;
         break;
@@ -449,17 +470,55 @@ static void parse_commandline(int argc, char *argv[], Options *op)
                                                        NV_OPTIONAL_BOOL_FALSE;
             break;
         case GLVND_GLX_CLIENT_OPTION:
-            op->glvnd_glx_client = boolval;
+            /* This option is no longer used */
+            if (!boolval) {
+                nv_error_msg("The --no-glvnd-glx-client option is no longer supported");
+                goto fail;
+            }
+
+            nv_warning_msg("The '--glvnd-glx-client' option is deprecated:  "
+                           "nvidia-installer will ignore this option.");
             break;
         case GLVND_EGL_CONFIG_FILE_PATH_OPTION:
             op->libglvnd_json_path = strval;
             break;
         case GLVND_EGL_CLIENT_OPTION:
-            op->glvnd_egl_client = boolval;
+            /* This option is no longer used */
+            if (!boolval) {
+                nv_error_msg("The --no-glvnd-egl-client option is no longer supported");
+                goto fail;
+            }
+
+            nv_warning_msg("The '--glvnd-egl-client' option is deprecated:  "
+                           "nvidia-installer will ignore this option.");
             break;
         case EGL_EXTERNAL_PLATFORM_CONFIG_FILE_PATH_OPTION:
             op->external_platform_json_path = strval;
             break;
+        case OVERRIDE_FILE_TYPE_DESTINATION_OPTION:
+            if (!assign_file_type_override(op, strval)) {
+                goto fail;
+            }
+            break;
+        case SKIP_DEPMOD_OPTION:
+            op->skip_depmod = TRUE;
+            break;
+        case SYSTEMD_OPTION:
+            op->use_systemd = boolval ? NV_OPTIONAL_BOOL_TRUE :
+                                        NV_OPTIONAL_BOOL_FALSE;
+            break;
+        case SYSTEMD_UNIT_PREFIX_OPTION:
+            op->systemd_unit_prefix = strval;
+            break;
+        case SYSTEMD_SLEEP_PREFIX_OPTION:
+            op->systemd_sleep_prefix = strval;
+            break;
+        case SYSTEMD_SYSCONF_PREFIX_OPTION:
+            op->systemd_sysconf_prefix = strval;
+            break;
+        case 'm':
+            op->kernel_module_build_directory_override = strval;
+            break;
         default:
             goto fail;
         }
@@ -583,6 +642,7 @@ int main(int argc, char *argv[])
     if (!find_system_utils(op)) goto done;
     if (!find_module_utils(op)) goto done;
     if (!check_selinux(op)) goto done;
+    if (!check_systemd(op)) goto done;
 
     /* check for X server properties based on the version of the server */
 
@@ -607,7 +667,8 @@ int main(int argc, char *argv[])
     /* uninstall */
 
     else if (op->uninstall) {
-        ret = uninstall_existing_driver(op, TRUE);
+        ret = uninstall_existing_driver(op, TRUE /* interactive */,
+                                        op->skip_depmod);
     }
 
     /* add this kernel */
