@@ -38,6 +38,8 @@ enum {
     XFREE86_PREFIX_OPTION = 1024,
     OPENGL_PREFIX_OPTION,
     OPENGL_LIBDIR_OPTION,
+    WINE_PREFIX_OPTION,
+    WINE_LIBDIR_OPTION,
     KERNEL_INCLUDE_PATH_OPTION,
     KERNEL_INSTALL_PATH_OPTION,
     UNINSTALL_OPTION,
@@ -47,7 +49,6 @@ enum {
     LOG_FILE_NAME_OPTION,
     HELP_ARGS_ONLY_OPTION,
     TMPDIR_OPTION,
-    OPENGL_HEADERS_OPTION,
     NO_NVIDIA_MODPROBE_OPTION,
     INSTALLER_PREFIX_OPTION,
     FORCE_TLS_OPTION,
@@ -82,6 +83,7 @@ enum {
     NO_CC_VERSION_CHECK_OPTION,
     NO_DISTRO_SCRIPTS_OPTION,
     NO_OPENGL_FILES_OPTION,
+    NO_WINE_FILES_OPTION,
     KERNEL_MODULE_SOURCE_PREFIX_OPTION,
     KERNEL_MODULE_SOURCE_DIR_OPTION,
     NO_KERNEL_MODULE_SOURCE_OPTION,
@@ -96,6 +98,7 @@ enum {
     NO_CHECK_FOR_ALTERNATE_INSTALLS_OPTION,
     NO_UVM_OPTION,
     NO_DRM_OPTION,
+    NO_PEERMEM_OPTION,
     INSTALL_COMPAT32_LIBS_OPTION,
     X_SYSCONFIG_PATH_OPTION,
     FORCE_LIBGLX_INDIRECT,
@@ -105,6 +108,12 @@ enum {
     GLVND_EGL_CONFIG_FILE_PATH_OPTION,
     GLVND_EGL_CLIENT_OPTION,
     EGL_EXTERNAL_PLATFORM_CONFIG_FILE_PATH_OPTION,
+    OVERRIDE_FILE_TYPE_DESTINATION_OPTION,
+    SKIP_DEPMOD_OPTION,
+    SYSTEMD_OPTION,
+    SYSTEMD_UNIT_PREFIX_OPTION,
+    SYSTEMD_SLEEP_PREFIX_OPTION,
+    SYSTEMD_SYSCONF_PREFIX_OPTION,
 };
 
 static const NVGetoptOption __options[] = {
@@ -219,6 +228,16 @@ static const NVGetoptOption __options[] = {
       "depending on the installed Linux distribution.  Only under very rare "
       "circumstances should this option be used." },
 
+     { "wine-prefix", WINE_PREFIX_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+       "The prefix under which Wine components of the NVIDIA driver will be "
+       "installed.  The default is '" DEFAULT_WINE_PREFIX "'."},
+
+     { "wine-libdir", WINE_LIBDIR_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+       "The path relative to the Wine installation prefix under which Wine "
+       "components of the NVIDIA driver will be installed.  "
+       "The default is '<opengl-libdir>/" DEFAULT_WINE_LIBDIR_SUFFIX "', where "
+       "<opengl-libdir> is the path determined for --opengl-libdir above."},
+
 #if defined(NV_X86_64)
     { "compat32-chroot", COMPAT32_CHROOT_OPTION, NVGETOPT_STRING_ARGUMENT,
       NULL,
@@ -350,15 +369,6 @@ static const NVGetoptOption __options[] = {
     { "no-ncurses-color", 'c', NVGETOPT_OPTION_APPLIES_TO_NVIDIA_UNINSTALL,
       NULL, "Disable use of color in the ncurses user interface." },
 
-    { "opengl-headers", OPENGL_HEADERS_OPTION, 0, NULL,
-      "Normally, installation will not install NVIDIA's OpenGL "
-      "header files; the OpenGL header files packaged by the "
-      "Linux distribution or available from "
-      "http://www.opengl.org/registry/ should be preferred. "
-      "However, http://www.opengl.org/registry/ does not yet provide "
-      "a glx.h or gl.h.  Until that is resolved, NVIDIA's OpenGL "
-      "header files can still be chosen, through this installer option." },
-
     { "no-nvidia-modprobe", NO_NVIDIA_MODPROBE_OPTION, 0, NULL,
       "Skip installation of 'nvidia-modprobe', a setuid root utility which "
       "nvidia-installer installs by default.  nvidia-modprobe can be used by "
@@ -369,26 +379,6 @@ static const NVGetoptOption __options[] = {
       "needed if other means of loading the NVIDIA kernel module and creating "
       "the NVIDIA device files are unavailable." },
 
-#if defined(NV_TLS_TEST)
-    { "force-tls", FORCE_TLS_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
-      "NVIDIA's OpenGL libraries are compiled with one of two "
-      "different thread local storage (TLS) mechanisms: 'classic tls' "
-      "which is used on systems with glibc 2.2 or older, and 'new tls' "
-      "which is used on systems with tls-enabled glibc 2.3 or newer.  "
-      "nvidia-installer will select the OpenGL libraries appropriate "
-      "for your system; however, you may use this option to force the "
-      "installer to install one library type or another.  Valid values "
-      "for &FORCE-TLS& are 'new' and 'classic'." },
-
-#if defined(NV_X86_64)
-    { "force-tls-compat32", FORCE_TLS_COMPAT32_OPTION,
-      NVGETOPT_STRING_ARGUMENT, NULL,
-      "This option forces the installer to install a specific "
-      "32bit compatibility OpenGL TLS library; further details "
-      "can be found in the description of the '--force-tls' option." },
-#endif /* NV_X86_64 */
-#endif
-
     { "kernel-name", 'k', NVGETOPT_STRING_ARGUMENT, NULL,
       "Build and install the NVIDIA kernel module for the "
       "non-running kernel specified by &KERNEL-NAME& (&KERNEL-NAME& "
@@ -442,7 +432,7 @@ static const NVGetoptOption __options[] = {
       "existing, possibly conflicting kernel modules.  This can be "
       "useful in some DEBUG environments.  If you use this option, you "
       "must be careful to ensure that a NVIDIA kernel module matching "
-      "this driver version is installed seperately." },
+      "this driver version is installed separately." },
 
     { "no-x-check", NO_X_CHECK_OPTION, 0, NULL,
       "Do not abort the installation if nvidia-installer detects that "
@@ -524,6 +514,9 @@ static const NVGetoptOption __options[] = {
     { "no-opengl-files", NO_OPENGL_FILES_OPTION, 0, NULL,
       "Do not install any of the OpenGL-related driver files." },
 
+    { "no-wine-files", NO_WINE_FILES_OPTION, 0, NULL,
+      "Do not install any of the Wine driver files." },
+
     { "kernel-module-source-prefix", KERNEL_MODULE_SOURCE_PREFIX_OPTION, 
        NVGETOPT_STRING_ARGUMENT, NULL,
       "Specify a path where the source directory for the kernel module will "
@@ -612,6 +605,12 @@ static const NVGetoptOption __options[] = {
       "should only be used to work around failures to build or install the "
       "nvidia-drm kernel module on systems that do not need these features." },
 
+    { "no-peermem", NO_PEERMEM_OPTION, 0, NULL,
+      "Do not install the nvidia-peermem kernel module. This kernel module "
+      "provides support for peer-to-peer memory sharing with Mellanox HCAs "
+      "(Host Channel Adapters) via GPUDirect RDMA (Remote Direct Memory "
+      "Access)." },
+
     { "concurrency-level", 'j', NVGETOPT_INTEGER_ARGUMENT, NULL,
       "Set the concurrency level for operations such as building the kernel "
       "module which may be parallelized on SMP systems. By default, this will "
@@ -621,47 +620,26 @@ static const NVGetoptOption __options[] = {
       "a higher level on the command line will override this limit." },
 
     { "force-libglx-indirect", FORCE_LIBGLX_INDIRECT, 0, NULL,
-      "If the package includes a libglvnd-based OpenGL library, then always "
-      "install a libGLX_indirect.so.0 symlink, overwriting one if it exists." },
+      "Always install a libGLX_indirect.so.0 symlink, overwriting one if it "
+      "exists." },
     { "no-libglx-indirect", NO_LIBGLX_INDIRECT, 0, NULL,
       "Do not install a libGLX_indirect.so.0 symlink." },
 
     { "install-libglvnd", INSTALL_LIBGLVND_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
-      "If the package includes a libglvnd-based OpenGL library, then it will "
-      "try to determine whether the libglvnd libraries are already available, "
-      "and will install them if they're not. Use --install-libglvnd to always "
-      "install the libglvnd libraries, overwriting any that already exist. "
-      "Use --no-install-libglvnd to exclude the libglvnd libraries, even if "
-      "they appear to be missing." },
-
-    { "glvnd-glx-client", GLVND_GLX_CLIENT_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
-      "By default, the NVIDIA GLX driver will be installed with the new "
-      "GLVND architecture, to support coexisting with other GLVND-compliant "
-      "GLX drivers. However, some applications which do not conform to "
-      "the Linux OpenGL ABI may not be fully compatible with a GLVND-based "
-      "GLX driver. The --no-glvnd-glx-client option will select a "
-      "non-GLVND GLX client library (libGL.so.1), which may help to avoid "
-      "compatibility issues with such applications."
-    },
+      "The installer will try to determine whether the libglvnd libraries are "
+      "already available, and will install them if they're not. Use "
+      "--install-libglvnd to always install the libglvnd libraries, "
+      "overwriting any that already exist.  Use --no-install-libglvnd to "
+      "exclude the libglvnd libraries, even if they appear to be missing." },
 
     { "glvnd-egl-config-path", GLVND_EGL_CONFIG_FILE_PATH_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
-      "If the package includes a libglvnd-based EGL library, then install the "
-      "EGL vendor library config file to this directory. If the libglvnd "
-      "libraries are already present, then by default the installer will try "
-      "to determine the path by running `pkg-config --variable=datadir "
-      "libglvnd`. If that fails, then it will default to "
+      "Install the EGL vendor library config file to this directory. If the "
+      "libglvnd libraries are already present, then by default the installer "
+      "will try to determine the path by running `pkg-config "
+      "--variable=datadir libglvnd`. If that fails, then it will default to "
       DEFAULT_GLVND_EGL_JSON_PATH "."
     },
 
-    { "glvnd-egl-client", GLVND_EGL_CLIENT_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
-      "By default, the NVIDIA EGL driver will be installed with the new "
-      "GLVND architecture, to support coexisting with other GLVND-compliant "
-      "EGL drivers. However, some applications may not be fully compatible "
-      "with a GLVND-based EGL driver. The --no-glvnd-egl-client option will "
-      "select a non-GLVND EGL client library, which may help to avoid "
-      "compatibility issues with such applications."
-    },
-
     { "egl-external-platform-config-path",
       EGL_EXTERNAL_PLATFORM_CONFIG_FILE_PATH_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
       "If the package includes an EGL external platform library, then install "
@@ -669,6 +647,60 @@ static const NVGetoptOption __options[] = {
       "Defaults to " DEFAULT_EGL_EXTERNAL_PLATFORM_JSON_PATH "."
     },
 
+    { "override-file-type-destination",
+      OVERRIDE_FILE_TYPE_DESTINATION_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+      "Override the default destination for a file type. This option takes an "
+      "argument in the form of '<FILE_TYPE>:<destination>', where <FILE_TYPE> "
+      "is a file type from the installer .manifest file, and <destination> is "
+      "an absolute path to the directory where files of that type should be "
+      "installed. This option may be given multiple times in order to override "
+      "the destinations for multiple file types. Use of this option takes "
+      "precedence over any other options that might otherwise influence the "
+      "destination of the specified file type."
+    },
+
+    { "skip-depmod",
+      SKIP_DEPMOD_OPTION, NVGETOPT_OPTION_APPLIES_TO_NVIDIA_UNINSTALL, NULL,
+      "Don't run the depmod(1) utility after modifying kernel modules.  This "
+      "should only be used in cases where depmod(1) will be run separately after "
+      "running nvidia-installer."
+    },
+
+    { "systemd", SYSTEMD_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
+      "By default, the installer will install systemd unit files if systemctl "
+      "is detected. Specifying --no-systemd will disable installation of "
+      "systemd units." },
+
+    { "systemd-unit-prefix",
+      SYSTEMD_UNIT_PREFIX_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+      "The path to which systemd unit files should be installed. By default, "
+      "the installer uses `pkg-config --variable=systemdsystemunitdir systemd` "
+      "to determine the location to install systemd unit files, or "
+      "'/usr/lib/systemd/system' if pkg-config is not available. Ignored if "
+      "--no-systemd is specified." },
+
+    { "systemd-sleep-prefix",
+      SYSTEMD_SLEEP_PREFIX_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+      "The path to which systemd-sleep script files should be installed. By "
+      "default, the installer uses `pkg-config --variable=systemdsleepdir "
+      "systemd` to determine the location to install systemd-sleep script "
+      "files, or '/usr/lib/systemd/system-sleep' if pkg-config is not "
+      "available. Ignored if --no-systemd is specified." },
+
+    { "systemd-sysconf-prefix",
+      SYSTEMD_SYSCONF_PREFIX_OPTION, NVGETOPT_STRING_ARGUMENT, NULL,
+      "The path to which systemd unit enable symlinks should be installed. By "
+      "default, the installer uses `pkg-config "
+      "--variable=systemdsystemconfdir systemd` to determine the location to "
+      "install these symlinks, or '/etc/systemd/system' if pkg-config is not "
+      "available. Ignored if --no-systemd is specified." },
+
+    { "kernel-module-build-directory", 'm',
+      NVGETOPT_STRING_ARGUMENT, NULL,
+      "The path within the driver package that contains the kernel module "
+      "build files is normally 'kernel/'.  This option can be used to "
+      "override this value." },
+
     /* Orphaned options: These options were in the long_options table in
      * nvidia-installer.c but not in the help. */
     { "debug",                    'd', 0, NULL,NULL },
@@ -686,6 +718,11 @@ static const NVGetoptOption __options[] = {
     { "no-runlevel-check", NO_RUNLEVEL_CHECK_OPTION, 0, NULL, NULL },
     { "install-vdpau-wrapper", INSTALL_VDPAU_WRAPPER_OPTION, NVGETOPT_IS_BOOLEAN, NULL, NULL },
     { "no-network", 'N', 0, NULL, NULL },
+    { "force-tls", FORCE_TLS_OPTION, NVGETOPT_STRING_ARGUMENT, NULL, NULL },
+    { "force-tls-compat32", FORCE_TLS_COMPAT32_OPTION, NVGETOPT_STRING_ARGUMENT,
+      NULL, NULL },
+    { "glvnd-glx-client", GLVND_GLX_CLIENT_OPTION, NVGETOPT_IS_BOOLEAN, NULL, NULL },
+    { "glvnd-egl-client", GLVND_EGL_CLIENT_OPTION, NVGETOPT_IS_BOOLEAN, NULL, NULL },
 
     { NULL, 0, 0, NULL, NULL },
 };
